/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.cli;

// Aether-related classes
import com.rempl.aether.AbstractAetherLoader;
import com.rempl.aether.Aether;
import com.rempl.aether.Resolver;

// API classes
import com.rempl.api.LoadingException;
import com.ymock.util.Logger;

// AETHER

// for file manipulations
import java.io.File;

// for file manipulations
import java.net.URL;
import java.net.URLClassLoader;

// collections
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Collection;

/**
 * Loader of an artifact into home directory of the executer.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: HomeLoader.java 1614 2011-07-10 19:05:43Z guard $
 * @see <a href="http://maven.apache.org/ref/3.0.3/maven-repository-metadata/repository-metadata.html">Maven Descriptor Reference</a>
 */
public final class HomeLoader extends AbstractAetherLoader {

    /**
     * URL class loader.
     */
    private static class PriviledgedUrlClassLoader
        implements PrivilegedAction<Object> {
        /**
         * URLs to be used by the classloader.
         */
        private final URL[] urls;
        /**
         * Public ctor.
         * @param newUrls URLs that should used
         */
        public PriviledgedUrlClassLoader(final URL[] newUrls) {
            this.urls = newUrls;
        }
        /**
         * {@inheritDoc}
         */
        @Override
        public Object run() {
            Thread.currentThread().setContextClassLoader(
                new URLClassLoader(
                    this.urls,
                    Thread.currentThread().getContextClassLoader()
                )
            );
            return null;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Resolver getResolver() {
        return new Aether();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void addToLoader(final URL[] urls) {
        AccessController.doPrivileged(new PriviledgedUrlClassLoader(urls));
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String rom(final Collection<File> jars) throws LoadingException {
        for (File jar : jars) {
            final String xml = this.readRomFromJar(jar);
            if (xml != null) {
                Logger.debug(
                    this,
                    "#rom(%d JARs): rom.xml loaded in %d bytes",
                    jars.size(),
                    xml.length()
                );
                return xml;
            }
        }
        throw new LoadingException("ROM.xml is not found");
    }

}
