/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.cli;

// for logging, and API access
import com.rempl.api.Model;
import com.rempl.api.ROM;
import com.rempl.api.Rempl;
import com.rempl.api.RemplFactory;
import com.rempl.api.Rempler;

import com.ymock.util.Logger;
import com.ymock.util.RuntimeProblem;

// for file listing
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Reader;
import java.io.Writer;

// CLI options
import org.apache.commons.cli.CommandLine;

/**
 * CLI request.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Main.java 42 2010-12-17 16:16:55Z yegor256@yahoo.com $
 */
class Request {

    /**
     * "rom" option that specifies the location of
     * <tt>rom.xml</tt> file.
     */
    public static final String OPT_ROM = "rom";

    /**
     * "xmi" option that specifies the location of
     * XMI file to be generated.
     */
    public static final String OPT_XMI = "xmi";

    /**
     * "report" option that specifies the location of
     * directory where reports will be saved to.
     */
    public static final String OPT_DIR = "dir";

    /**
     * The command line with options from CLI.
     */
    private CommandLine cmd;

    /**
     * Public ctor.
     * @param cline Command line
     * @see Main#main(String[])
     */
    public Request(final CommandLine cline) {
        this.cmd = cline;
    }

    /**
     * Read REMPL meta-info.
     */
    public void read() {
        Logger.info(this, "Reading started");
        final Rempler rempler = this.rempler();
        Writer writer;
        try {
            writer = new FileWriter(this.xmi());
        } catch (java.io.IOException ex) {
            throw RuntimeProblem.make(ex);
        }
        Exception failure = null;
        try {
            rempler.read().save(writer);
        } catch (java.io.IOException ex) {
            failure = ex;
        }
        try {
            writer.close();
        } catch (java.io.IOException ex) {
            failure = ex;
        }
        if (failure != null) {
            throw RuntimeProblem.make(failure);
        }
        Logger.info(this, "Reading finished");
    }

    /**
     * Build report.
     */
    public void report() {
        Logger.info(this, "Reporting started");
        final Rempler rempler = this.rempler();
        final Model model = rempler.emptyModel();
        Reader reader;
        try {
            reader = new FileReader(this.xmi());
        } catch (java.io.IOException ex) {
            throw RuntimeProblem.make(ex);
        }
        Exception failure = null;
        try {
            model.load(reader);
            rempler.report(model, this.dir());
        } catch (java.io.IOException ex) {
            failure = ex;
        }
        try {
            reader.close();
        } catch (java.io.IOException ex) {
            failure = ex;
        }
        if (failure != null) {
            throw RuntimeProblem.make(failure);
        }
        Logger.info(this, "Reporting finished");
    }

    /**
     * Instantiate Rempler.
     * @return The rempler to use later
     */
    public Rempler rempler() {
        final RemplFactory factory = Rempl.getFactory();
        factory.setLoader(new HomeLoader());
        if (!this.cmd.hasOption(this.OPT_ROM)) {
            throw RuntimeProblem.make(
                "--%s option is not set, where to find ROM.xml?",
                this.OPT_ROM
            );
        }
        final File file = new File(this.cmd.getOptionValue(this.OPT_ROM));
        ROM rom;
        try {
            rom = new ROM(file);
        } catch (com.rempl.api.ROMException ex) {
            throw RuntimeProblem.make(ex);
        }
        return factory.makeRempler(rom);
    }

    /**
     * Get location of XMI file.
     * @return The file
     */
    public File xmi() {
        if (!this.cmd.hasOption(this.OPT_XMI)) {
            throw RuntimeProblem.make(
                "--%s option is not set, where to save/load XMI?",
                this.OPT_XMI
            );
        }
        return new File(this.cmd.getOptionValue(this.OPT_XMI));
    }

    /**
     * Get location of reporting directory.
     * @return The directory
     */
    public File dir() {
        if (!this.cmd.hasOption(this.OPT_DIR)) {
            throw RuntimeProblem.make(
                "--%s option is not set, where to report to?",
                this.OPT_DIR
            );
        }
        final File dir = new File(this.cmd.getOptionValue(this.OPT_DIR));
        if (dir.mkdirs()) {
            Logger.info(this, "Directory '%s' created", dir);
        }
        return dir;
    }

}
